import * as React from "react";
import SFilter from "./comps/SFilter";
import STable from "./comps/STable";
import SModal from "./comps/SModal";
import { getList } from "./lib/api";
import { dealParams, dealRangePickerParams } from "./utils/params";
import { isObj } from "./utils/tool";
import verifyParams from "./lib/verifyParams";
import useColumns from "./hooks/useColumns";
import { compDidMount } from "./lifeCycleHooks/lifeCycle";
const { useState } = React;

export default function SPage(props: any) {
  const {
    url,
    filters,
    tableColumns,
    searchHandler,
    isDealRangePicker = true,
    initialData,
    initialSearch = true,
    tableProps = {},
    rowKey
  } = props;
  tableProps["rowKey"] = rowKey;
  const [formData, setFormData] = useState(initialData);
  const [tableData, setTableData] = useState(props.tableData);
  const [pageSize, setPageSize] = useState(10);
  const [pageNum, setPageNum] = useState(1);
  const [totalNum, setTotalNum] = useState(0);
  const [modalChildren, setModalChildren] = useState(null);
  const [modalProps, setModalProps] = useState({});
  const [modalVisible, setModalVisible] = useState(false);
  const [searchLoading, setSearchLoading] = useState(false);
  const requiredParams: string[] = [];
  filters.forEach(({ required, key }: any) => {
    if (required) requiredParams.push(key);
  });
  const getTableList = async (opt: any) => {
    if (isDealRangePicker) opt = dealRangePickerParams(opt);
    setSearchLoading(true);
    try {
      const {
        data: { list, total_num }
      } = await getList(url, opt);
      setTotalNum(total_num);
      setTableData(list || []);
    } finally {
      setSearchLoading(false);
    }
  };
  const _searchHandler = () => {
    let opt = formData;
    const handlerRes = searchHandler && searchHandler(opt);
    isObj(handlerRes) && (opt = handlerRes);
    opt = dealParams(opt);
    if (!verifyParams(opt, requiredParams)) return;
    setPageNum(1);
    return getTableList({
      ...opt,
      page_size: pageSize,
      page_num: 1
    });
  };
  const openModal = (
    record: any,
    modalRender: Function,
    modalProps: any = {}
  ) => {
    setModalVisible(true);
    setModalChildren(modalRender && modalRender(record));
    setModalProps(modalProps);
  };
  const closeModal = () => {
    setModalVisible(false);
  };

  const pageChange = (pageNum: any, pageSize: any) => {
    setPageNum(pageNum);
    setPageSize(pageSize);
    getTableList({
      ...formData,
      page_size: pageSize,
      page_num: pageNum
    });
  };
  const columns = useColumns(filters, tableColumns, openModal);
  if (initialSearch) compDidMount(_searchHandler);
  return (
    <div>
      <SFilter
        updateFormData={setFormData}
        searchBtnHandler={_searchHandler}
        searchLoading={searchLoading}
        {...props}
      />
      {props.children}
      <STable
        tableData={tableData}
        tableColumns={columns}
        tableProps={tableProps}
        pageChange={pageChange}
        pageSize={pageSize}
        pageNum={pageNum}
        totalNum={totalNum}
        loading={searchLoading}
      />
      <SModal closeModal={closeModal} props={modalProps} visible={modalVisible}>
        {modalChildren}
      </SModal>
    </div>
  );
}
SPage.defaultProps = {
  filters: [],
  tableColumns: []
};
