import * as React from "react";
import { Form, Card } from "antd";
import { isFn, isPromise } from "../utils/tool";
import renderOptions from "../lib/renderOptions";
import SButton from "./SButton";
import compMap from "../SCompMap/compMap";
const { useState } = React;
const defaultItemWidth = {
  RangePicker: 400
};

export default function SFilter({
  initialData = {},
  filters,
  formProps = {},
  searchBtnHandler,
  buttons = [],
  onChange: formChange,
  updateFormData,
  searchLoading
}: any) {
  const [formData, setFormData] = useState({ ...initialData });
  const [itemChildren, setItemChildren] = useState({});
  const requiredParams: any = [];
  const filterItemChange = (v: any, key: string, onChange: Function) => {
    if (v && v.target) v = v.target.value;
    const res = {
      ...formData,
      [key]: v
    };
    setFormData(res);
    updateFormData(res);
    if (isFn(onChange)) onChange(v);
    if (isFn(formChange)) formChange(res);
  };
  return (
    <Card>
      <Form layout="inline" {...formProps}>
        {filters.map(
          ({
            Comp,
            type = "Input",
            label,
            key,
            formItemProps,
            render,
            onChange,
            options,
            style = {},
            props,
            required
          }: any) => {
            if (required) requiredParams.push(key);
            style.width = defaultItemWidth[type] || 300;
            const Item = compMap[type];
            const Children = render && render();
            if (!options && isPromise(Children) && !itemChildren[key]) {
              Children.then((Child: any) => {
                setItemChildren({
                  ...itemChildren,
                  [key]: Child
                });
              }).catch((err: any) => {
                console.log(`get ${key} options error :` + err);
              });
            }
            return Item ? (
              <Form.Item
                required={required}
                key={label}
                label={label}
                {...formItemProps}
              >
                {Comp ? (
                  isFn(Comp) ? (
                    <Comp />
                  ) : (
                    Comp
                  )
                ) : (
                  <Item
                    allowClear={!required}
                    value={formData[key]}
                    onChange={(e: any) => {
                      filterItemChange(e, key, onChange);
                    }}
                    style={style}
                    {...props}
                  >
                    {options
                      ? renderOptions(options)
                      : isPromise(Children)
                      ? itemChildren[key]
                      : Children}
                  </Item>
                )}
              </Form.Item>
            ) : null;
          }
        )}
        <Form.Item>
          {
            <SButton
              handler={searchBtnHandler}
              label="Search"
              type="primary"
              loading={searchLoading}
              props={{
                htmlType: "submit"
              }}
            />
          }
          {buttons.map(({ Comp, label, key, ...rest }: any) => {
            if (Comp) {
              return isFn(Comp) ? <Comp key={key} /> : Comp;
            }
            return <SButton key={label} label={label} {...rest} />;
          })}
        </Form.Item>
      </Form>
    </Card>
  );
}
